@@ -277,8 +277,8 @@ final class SchemaUtil {
     }
   }
 
-  public static void writeMessageList(int fieldNumber, List<?> value, Writer writer, Schema schema)
-      throws IOException {
+  public static void writeMessageList(
+      int fieldNumber, List<?> value, Writer writer, Schema<?> schema) throws IOException {
     if (value != null && !value.isEmpty()) {
       writer.writeMessageList(fieldNumber, value, schema);
     }
@@ -300,7 +300,7 @@ final class SchemaUtil {
     }
   }
 
-  public static void writeGroupList(int fieldNumber, List<?> value, Writer writer, Schema schema)
+  public static void writeGroupList(int fieldNumber, List<?> value, Writer writer, Schema<?> schema)
       throws IOException {
     if (value != null && !value.isEmpty()) {
       writer.writeGroupList(fieldNumber, value, schema);
@@ -644,14 +644,31 @@ final class SchemaUtil {
     return size;
   }
 
-  static int computeSizeMessage(int fieldNumber, Object value, Schema schema) {
+  static int computeSizeMessage(int fieldNumber, Object value, Schema<?> schema) {
     if (value instanceof LazyFieldLite) {
       return CodedOutputStream.computeLazyFieldSize(fieldNumber, (LazyFieldLite) value);
     } else {
-      return CodedOutputStream.computeMessageSize(fieldNumber, (MessageLite) value, schema);
+      return computeMessageSize(fieldNumber, (AbstractMessageLite) value, schema);
     }
   }
 
+  /**
+   * Compute the number of bytes that would be needed to encode an embedded message field, including
+   * tag.
+   */
+  @SuppressWarnings("rawtypes")
+  static int computeMessageSize(
+      final int fieldNumber, final AbstractMessageLite value, final Schema schema) {
+    return CodedOutputStream.computeTagSize(fieldNumber) + computeMessageSizeNoTag(value, schema);
+  }
+
+  /** Compute the number of bytes that would be needed to encode an embedded message field. */
+  @SuppressWarnings("rawtypes")
+  static int computeMessageSizeNoTag(final AbstractMessageLite value, final Schema schema) {
+    return CodedOutputStream.computeLengthDelimitedFieldSize(
+        ((AbstractMessageLite) value).getSerializedSize(schema));
+  }
+
   static int computeSizeMessageList(int fieldNumber, List<?> list) {
     final int length = list.size();
     if (length == 0) {
@@ -669,7 +686,7 @@ final class SchemaUtil {
     return size;
   }
 
-  static int computeSizeMessageList(int fieldNumber, List<?> list, Schema schema) {
+  static int computeSizeMessageList(int fieldNumber, List<?> list, Schema<?> schema) {
     final int length = list.size();
     if (length == 0) {
       return 0;
@@ -680,7 +697,7 @@ final class SchemaUtil {
       if (value instanceof LazyFieldLite) {
         size += CodedOutputStream.computeLazyFieldSizeNoTag((LazyFieldLite) value);
       } else {
-        size += CodedOutputStream.computeMessageSizeNoTag((MessageLite) value, schema);
+        size += computeMessageSizeNoTag((AbstractMessageLite) value, schema);
       }
     }
     return size;
@@ -710,7 +727,7 @@ final class SchemaUtil {
     return size;
   }
 
-  static int computeSizeGroupList(int fieldNumber, List<MessageLite> list, Schema schema) {
+  static int computeSizeGroupList(int fieldNumber, List<MessageLite> list, Schema<?> schema) {
     final int length = list.size();
     if (length == 0) {
       return 0;
