@@ -11,9 +11,6 @@
 #import "GPBWireFormat.h"
 
 #import "GPBArray.h"
-#import "GPBUnknownFieldSet.h"
-
-@class GPBUnknownFieldSet;
 
 NS_ASSUME_NONNULL_BEGIN
 
@@ -587,36 +584,6 @@ __attribute__((objc_subclassing_restricted))
 
 // clang-format on
 
-/**
- * Write a GPBUnknownFieldSet for the given field number.
- *
- * @param fieldNumber The field number assigned to the value.
- * @param value       The value to write out.
- **/
-- (void)writeUnknownGroup:(int32_t)fieldNumber
-                    value:(GPBUnknownFieldSet *)value
-    __attribute__((deprecated("GPBUnknownFieldSet is going away.")));
-
-/**
- * Write an array of GPBUnknownFieldSet for the given field number.
- *
- * @param fieldNumber The field number assigned to the values.
- * @param values      The values to write out.
- **/
-- (void)writeUnknownGroupArray:(int32_t)fieldNumber
-                        values:(NSArray<GPBUnknownFieldSet *> *)values
-    __attribute__((deprecated("GPBUnknownFieldSet is going away.")));
-
-/**
- * Write a GPBUnknownFieldSet without any tag (but does write the endGroup tag).
- *
- * @param fieldNumber The field number assigned to the value.
- * @param value       The value to write out.
- **/
-- (void)writeUnknownGroupNoTag:(int32_t)fieldNumber
-                         value:(GPBUnknownFieldSet *)value
-    __attribute__((deprecated("GPBUnknownFieldSet is going away.")));
-
 /**
 Write a MessageSet extension field to the stream. For historical reasons,
 the wire format differs from normal fields.
