@@ -14,6 +14,7 @@
 #include "google/protobuf/compiler/command_line_interface_tester.h"
 #include "google/protobuf/cpp_features.pb.h"
 
+
 namespace google {
 namespace protobuf {
 namespace compiler {
@@ -255,8 +256,8 @@ TEST_F(CppGeneratorTest, StringTypeCordNotForExtension) {
       "--experimental_editions foo.proto");
 
   ExpectErrorSubstring(
-      "Extension bar specifies Cord type which is not supported for "
-      "extensions.");
+      "Extension bar specifies CORD string type which is not supported for "
+      "extensions");
 }
 
 TEST_F(CppGeneratorTest, InheritedStringTypeCordNotForExtension) {
@@ -280,7 +281,7 @@ TEST_F(CppGeneratorTest, InheritedStringTypeCordNotForExtension) {
   ExpectNoErrors();
 }
 
-TEST_F(CppGeneratorTest, CtypeOnNoneStringFieldTest) {
+TEST_F(CppGeneratorTest, CtypeOnNonStringFieldTest) {
   CreateTempFile("foo.proto",
                  R"schema(
     edition = "2023";
@@ -290,8 +291,8 @@ TEST_F(CppGeneratorTest, CtypeOnNoneStringFieldTest) {
   RunProtoc(
       "protocol_compiler --proto_path=$tmpdir --cpp_out=$tmpdir foo.proto");
   ExpectErrorSubstring(
-      "Field Foo.bar specifies ctype, but is not "
-      "a string nor bytes field.");
+      "Field Foo.bar specifies string_type, but is not a string nor bytes "
+      "field.");
 }
 
 TEST_F(CppGeneratorTest, CtypeOnExtensionTest) {
@@ -307,9 +308,11 @@ TEST_F(CppGeneratorTest, CtypeOnExtensionTest) {
   RunProtoc(
       "protocol_compiler --proto_path=$tmpdir --cpp_out=$tmpdir foo.proto");
   ExpectErrorSubstring(
-      "Extension bar specifies Cord type which is "
-      "not supported for extensions.");
+      "Extension bar specifies CORD string type which is not supported for "
+      "extensions");
 }
+
+
 }  // namespace
 }  // namespace cpp
 }  // namespace compiler
