@@ -1,37 +1,18 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
 //  Sanjay Ghemawat, Jeff Dean, and others.
 
+#include "google/protobuf/compiler/importer.h"
+
+#include <string>
+
 #ifdef _MSC_VER
 #include <direct.h>
 #else
@@ -42,18 +23,25 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include <algorithm>
 #include <memory>
+#include <vector>
 
-#include <google/protobuf/compiler/importer.h>
-#include <google/protobuf/compiler/parser.h>
-#include <google/protobuf/io/tokenizer.h>
-#include <google/protobuf/io/zero_copy_stream_impl.h>
-#include <google/protobuf/stubs/strutil.h>
-#include <google/protobuf/io/io_win32.h>
+#include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_join.h"
+#include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "google/protobuf/compiler/parser.h"
+#include "google/protobuf/io/tokenizer.h"
+#include "google/protobuf/io/zero_copy_stream_impl.h"
 
 #ifdef _WIN32
-#include <ctype.h>
+#include "absl/strings/str_replace.h"
+#include "google/protobuf/io/io_win32.h"
+#endif
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+#include "absl/strings/ascii.h"
 #endif
 
 namespace google {
@@ -68,11 +56,11 @@ using google::protobuf::io::win32::open;
 #endif
 
 // Returns true if the text looks like a Windows-style absolute path, starting
-// with a drive letter.  Example:  "C:\foo".  TODO(kenton):  Share this with
+// with a drive letter.  Example:  "C:\foo".  TODO:  Share this with
 // copy in command_line_interface.cc?
-static bool IsWindowsAbsolutePath(const std::string& text) {
+static bool IsWindowsAbsolutePath(absl::string_view text) {
 #if defined(_WIN32) || defined(__CYGWIN__)
-  return text.size() >= 3 && text[1] == ':' && isalpha(text[0]) &&
+  return text.size() >= 3 && text[1] == ':' && absl::ascii_isalpha(text[0]) &&
          (text[2] == '/' || text[2] == '\\') && text.find_last_of(':') == 1;
 #else
   return false;
@@ -98,13 +86,21 @@ class SourceTreeDescriptorDatabase::SingleFileErrorCollector
   bool had_errors() { return had_errors_; }
 
   // implements ErrorCollector ---------------------------------------
-  void AddError(int line, int column, const std::string& message) override {
+  void RecordError(int line, int column, absl::string_view message) override {
     if (multi_file_error_collector_ != nullptr) {
-      multi_file_error_collector_->AddError(filename_, line, column, message);
+      multi_file_error_collector_->RecordError(filename_, line, column,
+                                               message);
     }
     had_errors_ = true;
   }
 
+  void RecordWarning(int line, int column, absl::string_view message) override {
+    if (multi_file_error_collector_ != nullptr) {
+      multi_file_error_collector_->RecordWarning(filename_, line, column,
+                                                 message);
+    }
+  }
+
  private:
   std::string filename_;
   MultiFileErrorCollector* multi_file_error_collector_;
@@ -131,7 +127,7 @@ SourceTreeDescriptorDatabase::SourceTreeDescriptorDatabase(
 
 SourceTreeDescriptorDatabase::~SourceTreeDescriptorDatabase() {}
 
-bool SourceTreeDescriptorDatabase::FindFileByName(const std::string& filename,
+bool SourceTreeDescriptorDatabase::FindFileByName(StringViewArg filename,
                                                   FileDescriptorProto* output) {
   std::unique_ptr<io::ZeroCopyInputStream> input(source_tree_->Open(filename));
   if (input == nullptr) {
@@ -140,8 +136,8 @@ bool SourceTreeDescriptorDatabase::FindFileByName(const std::string& filename,
       return true;
     }
     if (error_collector_ != nullptr) {
-      error_collector_->AddError(filename, -1, 0,
-                                 source_tree_->GetLastErrorMessage());
+      error_collector_->RecordError(filename, -1, 0,
+                                    source_tree_->GetLastErrorMessage());
     }
     return false;
   }
@@ -164,12 +160,12 @@ bool SourceTreeDescriptorDatabase::FindFileByName(const std::string& filename,
 }
 
 bool SourceTreeDescriptorDatabase::FindFileContainingSymbol(
-    const std::string& symbol_name, FileDescriptorProto* output) {
+    StringViewArg symbol_name, FileDescriptorProto* output) {
   return false;
 }
 
 bool SourceTreeDescriptorDatabase::FindFileContainingExtension(
-    const std::string& containing_type, int field_number,
+    StringViewArg containing_type, int field_number,
     FileDescriptorProto* output) {
   return false;
 }
@@ -183,10 +179,10 @@ SourceTreeDescriptorDatabase::ValidationErrorCollector::
 SourceTreeDescriptorDatabase::ValidationErrorCollector::
     ~ValidationErrorCollector() {}
 
-void SourceTreeDescriptorDatabase::ValidationErrorCollector::AddError(
-    const std::string& filename, const std::string& element_name,
+void SourceTreeDescriptorDatabase::ValidationErrorCollector::RecordError(
+    absl::string_view filename, absl::string_view element_name,
     const Message* descriptor, ErrorLocation location,
-    const std::string& message) {
+    absl::string_view message) {
   if (owner_->error_collector_ == nullptr) return;
 
   int line, column;
@@ -196,13 +192,13 @@ void SourceTreeDescriptorDatabase::ValidationErrorCollector::AddError(
   } else {
     owner_->source_locations_.Find(descriptor, location, &line, &column);
   }
-  owner_->error_collector_->AddError(filename, line, column, message);
+  owner_->error_collector_->RecordError(filename, line, column, message);
 }
 
-void SourceTreeDescriptorDatabase::ValidationErrorCollector::AddWarning(
-    const std::string& filename, const std::string& element_name,
+void SourceTreeDescriptorDatabase::ValidationErrorCollector::RecordWarning(
+    absl::string_view filename, absl::string_view element_name,
     const Message* descriptor, ErrorLocation location,
-    const std::string& message) {
+    absl::string_view message) {
   if (owner_->error_collector_ == nullptr) return;
 
   int line, column;
@@ -212,7 +208,7 @@ void SourceTreeDescriptorDatabase::ValidationErrorCollector::AddWarning(
   } else {
     owner_->source_locations_.Find(descriptor, location, &line, &column);
   }
-  owner_->error_collector_->AddWarning(filename, line, column, message);
+  owner_->error_collector_->RecordWarning(filename, line, column, message);
 }
 
 // ===================================================================
@@ -231,14 +227,11 @@ const FileDescriptor* Importer::Import(const std::string& filename) {
   return pool_.FindFileByName(filename);
 }
 
-void Importer::AddUnusedImportTrackFile(const std::string& file_name,
-                                        bool is_error) {
-  pool_.AddUnusedImportTrackFile(file_name, is_error);
+void Importer::AddDirectInputFile(absl::string_view file_name, bool is_error) {
+  pool_.AddDirectInputFile(file_name, is_error);
 }
 
-void Importer::ClearUnusedImportTrackFiles() {
-  pool_.ClearUnusedImportTrackFiles();
-}
+void Importer::ClearDirectInputFiles() { pool_.ClearDirectInputFiles(); }
 
 
 // ===================================================================
@@ -251,68 +244,58 @@ DiskSourceTree::DiskSourceTree() {}
 
 DiskSourceTree::~DiskSourceTree() {}
 
-static inline char LastChar(const std::string& str) {
-  return str[str.size() - 1];
-}
-
 // Given a path, returns an equivalent path with these changes:
 // - On Windows, any backslashes are replaced with forward slashes.
 // - Any instances of the directory "." are removed.
 // - Any consecutive '/'s are collapsed into a single slash.
 // Note that the resulting string may be empty.
 //
-// TODO(kenton):  It would be nice to handle "..", e.g. so that we can figure
+// TODO:  It would be nice to handle "..", e.g. so that we can figure
 //   out that "foo/bar.proto" is inside "baz/../foo".  However, if baz is a
 //   symlink or doesn't exist, then things get complicated, and we can't
 //   actually determine this without investigating the filesystem, probably
 //   in non-portable ways.  So, we punt.
 //
-// TODO(kenton):  It would be nice to use realpath() here except that it
+// TODO:  It would be nice to use realpath() here except that it
 //   resolves symbolic links.  This could cause problems if people place
 //   symbolic links in their source tree.  For example, if you executed:
 //     protoc --proto_path=foo foo/bar/baz.proto
 //   then if foo/bar is a symbolic link, foo/bar/baz.proto will canonicalize
 //   to a path which does not appear to be under foo, and thus the compiler
 //   will complain that baz.proto is not inside the --proto_path.
-static std::string CanonicalizePath(std::string path) {
+static std::string CanonicalizePath(absl::string_view path) {
 #ifdef _WIN32
   // The Win32 API accepts forward slashes as a path delimiter even though
   // backslashes are standard.  Let's avoid confusion and use only forward
   // slashes.
-  if (HasPrefixString(path, "\\\\")) {
+  std::string path_str;
+  if (absl::StartsWith(path, "\\\\")) {
     // Avoid converting two leading backslashes.
-    path = "\\\\" + StringReplace(path.substr(2), "\\", "/", true);
+    path_str = absl::StrCat("\\\\",
+                            absl::StrReplaceAll(path.substr(2), {{"\\", "/"}}));
   } else {
-    path = StringReplace(path, "\\", "/", true);
+    path_str = absl::StrReplaceAll(path, {{"\\", "/"}});
   }
+  path = path_str;
 #endif
 
-  std::vector<std::string> canonical_parts;
-  std::vector<std::string> parts = Split(
-      path, "/", true);  // Note:  Removes empty parts.
-  for (const std::string& part : parts) {
+  std::vector<absl::string_view> canonical_parts;
+  if (!path.empty() && path.front() == '/') canonical_parts.push_back("");
+  for (absl::string_view part : absl::StrSplit(path, '/', absl::SkipEmpty())) {
     if (part == ".") {
       // Ignore.
     } else {
       canonical_parts.push_back(part);
     }
   }
-  std::string result = Join(canonical_parts, "/");
-  if (!path.empty() && path[0] == '/') {
-    // Restore leading slash.
-    result = '/' + result;
-  }
-  if (!path.empty() && LastChar(path) == '/' && !result.empty() &&
-      LastChar(result) != '/') {
-    // Restore trailing slash.
-    result += '/';
-  }
-  return result;
+  if (!path.empty() && path.back() == '/') canonical_parts.push_back("");
+
+  return absl::StrJoin(canonical_parts, "/");
 }
 
-static inline bool ContainsParentReference(const std::string& path) {
-  return path == ".." || HasPrefixString(path, "../") ||
-         HasSuffixString(path, "/..") || path.find("/../") != std::string::npos;
+static inline bool ContainsParentReference(absl::string_view path) {
+  return path == ".." || absl::StartsWith(path, "../") ||
+         absl::EndsWith(path, "/..") || absl::StrContains(path, "/../");
 }
 
 // Maps a file from an old location to a new one.  Typically, old_prefix is
@@ -332,28 +315,28 @@ static inline bool ContainsParentReference(const std::string& path) {
 //   assert(!ApplyMapping("foo/bar", "baz", "qux", &result));
 //   assert(!ApplyMapping("foo/bar", "baz", "qux", &result));
 //   assert(!ApplyMapping("foobar", "foo", "baz", &result));
-static bool ApplyMapping(const std::string& filename,
-                         const std::string& old_prefix,
-                         const std::string& new_prefix, std::string* result) {
+static bool ApplyMapping(absl::string_view filename,
+                         absl::string_view old_prefix,
+                         absl::string_view new_prefix, std::string* result) {
   if (old_prefix.empty()) {
     // old_prefix matches any relative path.
     if (ContainsParentReference(filename)) {
       // We do not allow the file name to use "..".
       return false;
     }
-    if (HasPrefixString(filename, "/") || IsWindowsAbsolutePath(filename)) {
+    if (absl::StartsWith(filename, "/") || IsWindowsAbsolutePath(filename)) {
       // This is an absolute path, so it isn't matched by the empty string.
       return false;
     }
-    result->assign(new_prefix);
+    result->assign(std::string(new_prefix));
     if (!result->empty()) result->push_back('/');
-    result->append(filename);
+    result->append(std::string(filename));
     return true;
-  } else if (HasPrefixString(filename, old_prefix)) {
+  } else if (absl::StartsWith(filename, old_prefix)) {
     // old_prefix is a prefix of the filename.  Is it the whole filename?
     if (filename.size() == old_prefix.size()) {
       // Yep, it's an exact match.
-      *result = new_prefix;
+      *result = std::string(new_prefix);
       return true;
     } else {
       // Not an exact match.  Is the next character a '/'?  Otherwise,
@@ -370,14 +353,14 @@ static bool ApplyMapping(const std::string& filename,
       if (after_prefix_start != -1) {
         // Yep.  So the prefixes are directories and the filename is a file
         // inside them.
-        std::string after_prefix = filename.substr(after_prefix_start);
+        absl::string_view after_prefix = filename.substr(after_prefix_start);
         if (ContainsParentReference(after_prefix)) {
           // We do not allow the file name to use "..".
           return false;
         }
-        result->assign(new_prefix);
+        result->assign(std::string(new_prefix));
         if (!result->empty()) result->push_back('/');
-        result->append(after_prefix);
+        result->append(std::string(after_prefix));
         return true;
       }
     }
@@ -386,13 +369,14 @@ static bool ApplyMapping(const std::string& filename,
   return false;
 }
 
-void DiskSourceTree::MapPath(const std::string& virtual_path,
-                             const std::string& disk_path) {
-  mappings_.push_back(Mapping(virtual_path, CanonicalizePath(disk_path)));
+void DiskSourceTree::MapPath(absl::string_view virtual_path,
+                             absl::string_view disk_path) {
+  mappings_.push_back(
+      Mapping(std::string(virtual_path), CanonicalizePath(disk_path)));
 }
 
 DiskSourceTree::DiskFileToVirtualFileResult
-DiskSourceTree::DiskFileToVirtualFile(const std::string& disk_file,
+DiskSourceTree::DiskFileToVirtualFile(absl::string_view disk_file,
                                       std::string* virtual_file,
                                       std::string* shadowing_disk_file) {
   int mapping_index = -1;
@@ -436,14 +420,14 @@ DiskSourceTree::DiskFileToVirtualFile(const std::string& disk_file,
   return SUCCESS;
 }
 
-bool DiskSourceTree::VirtualFileToDiskFile(const std::string& virtual_file,
+bool DiskSourceTree::VirtualFileToDiskFile(absl::string_view virtual_file,
                                            std::string* disk_file) {
   std::unique_ptr<io::ZeroCopyInputStream> stream(
       OpenVirtualFile(virtual_file, disk_file));
   return stream != nullptr;
 }
 
-io::ZeroCopyInputStream* DiskSourceTree::Open(const std::string& filename) {
+io::ZeroCopyInputStream* DiskSourceTree::Open(absl::string_view filename) {
   return OpenVirtualFile(filename, nullptr);
 }
 
@@ -452,7 +436,7 @@ std::string DiskSourceTree::GetLastErrorMessage() {
 }
 
 io::ZeroCopyInputStream* DiskSourceTree::OpenVirtualFile(
-    const std::string& virtual_file, std::string* disk_file) {
+    absl::string_view virtual_file, std::string* disk_file) {
   if (virtual_file != CanonicalizePath(virtual_file) ||
       ContainsParentReference(virtual_file)) {
     // We do not allow importing of paths containing things like ".." or
@@ -479,7 +463,7 @@ io::ZeroCopyInputStream* DiskSourceTree::OpenVirtualFile(
       if (errno == EACCES) {
         // The file exists but is not readable.
         last_error_message_ =
-            "Read access is denied for file: " + temp_disk_file;
+            absl::StrCat("Read access is denied for file: ", temp_disk_file);
         return nullptr;
       }
     }
@@ -489,11 +473,11 @@ io::ZeroCopyInputStream* DiskSourceTree::OpenVirtualFile(
 }
 
 io::ZeroCopyInputStream* DiskSourceTree::OpenDiskFile(
-    const std::string& filename) {
+    absl::string_view filename) {
   struct stat sb;
   int ret = 0;
   do {
-    ret = stat(filename.c_str(), &sb);
+    ret = stat(std::string(filename).c_str(), &sb);
   } while (ret != 0 && errno == EINTR);
 #if defined(_WIN32)
   if (ret == 0 && sb.st_mode & S_IFDIR) {
@@ -508,7 +492,7 @@ io::ZeroCopyInputStream* DiskSourceTree::OpenDiskFile(
 #endif
   int file_descriptor;
   do {
-    file_descriptor = open(filename.c_str(), O_RDONLY);
+    file_descriptor = open(std::string(filename).c_str(), O_RDONLY);
   } while (file_descriptor < 0 && errno == EINTR);
   if (file_descriptor >= 0) {
     io::FileInputStream* result = new io::FileInputStream(file_descriptor);
