@@ -16,8 +16,10 @@
 #include <vector>
 
 #include "absl/container/btree_set.h"
+#include "absl/container/flat_hash_set.h"
 #include "absl/log/absl_check.h"
 #include "absl/log/absl_log.h"
+#include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "google/protobuf/compiler/code_generator.h"
@@ -29,10 +31,12 @@
 #include "google/protobuf/compiler/java/internal_helpers.h"
 #include "google/protobuf/compiler/java/lite/generator_factory.h"
 #include "google/protobuf/compiler/java/name_resolver.h"
+#include "google/protobuf/compiler/java/names.h"
 #include "google/protobuf/compiler/java/options.h"
 #include "google/protobuf/compiler/java/shared_code_generator.h"
 #include "google/protobuf/compiler/retention.h"
 #include "google/protobuf/compiler/versions.h"
+#include "google/protobuf/descriptor.h"
 #include "google/protobuf/descriptor.pb.h"
 #include "google/protobuf/descriptor_visitor.h"
 #include "google/protobuf/dynamic_message.h"
@@ -102,10 +106,20 @@ bool CollectExtensions(const Message& message, FieldDescriptorSet* extensions) {
   return true;
 }
 
+void CollectPublicDependencies(
+    const FileDescriptor* file,
+    absl::flat_hash_set<const FileDescriptor*>* dependencies) {
+  if (file == nullptr || !dependencies->insert(file).second) return;
+  for (int i = 0; file != nullptr && i < file->public_dependency_count(); i++) {
+    CollectPublicDependencies(file->public_dependency(i), dependencies);
+  }
+}
+
 // Finds all extensions for custom options in the given file descriptor with the
 // builder pool which resolves Java features instead of the generated pool.
-void CollectExtensions(const FileDescriptor& file,
-                       FieldDescriptorSet* extensions) {
+void CollectExtensions(const FileDescriptor& file, const Options& options,
+                       FieldDescriptorSet* extensions,
+                       FieldDescriptorSet* optional_extensions) {
   FileDescriptorProto file_proto = StripSourceRetentionOptions(file);
   std::string file_data;
   file_proto.SerializeToString(&file_data);
@@ -113,7 +127,7 @@ void CollectExtensions(const FileDescriptor& file,
       file_proto.GetDescriptor()->full_name());
 
   // descriptor.proto is not found in the builder pool, meaning there are no
-  // custom options.
+  // custom options or they are option imported and not reachable.
   if (file_proto_desc == nullptr) return;
 
   DynamicMessageFactory factory;
@@ -122,14 +136,33 @@ void CollectExtensions(const FileDescriptor& file,
   ABSL_CHECK(dynamic_file_proto.get() != nullptr);
   ABSL_CHECK(dynamic_file_proto->ParseFromString(file_data));
 
-  // Collect the extensions again from the dynamic message.
+  // Collect the extensions from the dynamic message.
   extensions->clear();
-  ABSL_CHECK(CollectExtensions(*dynamic_file_proto, extensions))
-      << "Found unknown fields in FileDescriptorProto when building "
-      << file_proto.name()
-      << ". It's likely that those fields are custom options, however, "
-         "those options cannot be recognized in the builder pool. "
-         "This normally should not happen. Please report a bug.";
+  // Unknown extensions are ok and expected in the case of option imports.
+  CollectExtensions(*dynamic_file_proto, extensions);
+
+  if (options.strip_nonfunctional_codegen) {
+    // Skip feature extensions, which are a visible (but non-functional)
+    // deviation between editions and legacy syntax.
+    absl::erase_if(*extensions, [](const FieldDescriptor* field) {
+      return field->containing_type()->full_name() == "google.protobuf.FeatureSet";
+    });
+  }
+
+  // Check against dependencies to handle option dependencies polluting pool.
+  absl::flat_hash_set<const FileDescriptor*> dependencies;
+  dependencies.insert(&file);
+  for (int i = 0; i < file.dependency_count(); i++) {
+    CollectPublicDependencies(file.dependency(i), &dependencies);
+  }
+  for (auto* extension : *extensions) {
+    if (!dependencies.contains(extension->file())) {
+      optional_extensions->insert(extension);
+    }
+  }
+  absl::erase_if(*extensions, [&](const FieldDescriptor* fieldDescriptor) {
+    return optional_extensions->contains(fieldDescriptor);
+  });
 }
 
 // Our static initialization methods can become very, very large.
@@ -252,6 +285,26 @@ bool FileGenerator::Validate(std::string* error) {
            "https://github.com/protocolbuffers/protobuf/blob/main/java/"
            "lite.md";
   }
+  google::protobuf::internal::VisitDescriptors(*file_, [&](const EnumDescriptor& enm) {
+    if (CheckLargeEnum(&enm) && enm.is_closed()) {
+      absl::StrAppend(
+          error, enm.full_name(),
+          " is a closed enum and can not be used with the large_enum feature.  "
+          "Please migrate to an open enum first, which is a better fit for "
+          "extremely large enums.\n");
+    }
+    absl::Status status = ValidateNestInFileClassFeature(enm);
+    if (!status.ok()) {
+      absl::StrAppend(error, status.message());
+    }
+  });
+
+  google::protobuf::internal::VisitDescriptors(*file_, [&](const Descriptor& message) {
+    absl::Status status = ValidateNestInFileClassFeature(message);
+    if (!status.ok()) {
+      absl::StrAppend(error, status.message());
+    }
+  });
 
   return error->empty();
 }
@@ -286,11 +339,14 @@ void FileGenerator::Generate(io::Printer* printer) {
     printer->Print("@com.google.protobuf.Internal.ProtoNonnullApi\n");
   }
   printer->Print(
-      "$deprecation$public final class $classname$ {\n"
+      "$deprecation$public final class $classname$ $extends${\n"
       "  private $ctor$() {}\n",
       "deprecation",
       file_->options().deprecated() ? "@java.lang.Deprecated " : "",
-      "classname", classname_, "ctor", classname_);
+      "classname", classname_, "ctor", classname_, "extends",
+      HasDescriptorMethods(file_, context_->EnforceLite())
+          ? "extends com.google.protobuf.GeneratedFile "
+          : "");
   printer->Annotate("classname", file_->name());
   printer->Indent();
 
@@ -338,17 +394,21 @@ void FileGenerator::Generate(io::Printer* printer) {
 
   // -----------------------------------------------------------------
 
-  if (!MultipleJavaFiles(file_, immutable_api_)) {
-    for (int i = 0; i < file_->enum_type_count(); i++) {
+  for (int i = 0; i < file_->enum_type_count(); i++) {
+    if (NestedInFileClass(*file_->enum_type(i), immutable_api_)) {
       generator_factory_->NewEnumGenerator(file_->enum_type(i))
           ->Generate(printer);
     }
-    for (int i = 0; i < file_->message_type_count(); i++) {
+  }
+  for (int i = 0; i < file_->message_type_count(); i++) {
+    if (NestedInFileClass(*file_->message_type(i), immutable_api_)) {
       message_generators_[i]->GenerateInterface(printer);
       message_generators_[i]->Generate(printer);
     }
-    if (HasGenericServices(file_, context_->EnforceLite())) {
-      for (int i = 0; i < file_->service_count(); i++) {
+  }
+  if (HasGenericServices(file_, context_->EnforceLite())) {
+    for (int i = 0; i < file_->service_count(); i++) {
+      if (NestedInFileClass(*file_->service(i), immutable_api_)) {
         std::unique_ptr<ServiceGenerator> generator(
             generator_factory_->NewServiceGenerator(file_->service(i)));
         generator->Generate(printer);
@@ -464,15 +524,8 @@ void FileGenerator::GenerateDescriptorInitializationCodeForImmutable(
   // of the FileDescriptor based on the builder-pool, then we can use
   // reflections to find all extension fields
   FieldDescriptorSet extensions;
-  CollectExtensions(*file_, &extensions);
-
-  if (options_.strip_nonfunctional_codegen) {
-    // Skip feature extensions, which are a visible (but non-functional)
-    // deviation between editions and legacy syntax.
-    absl::erase_if(extensions, [](const FieldDescriptor* field) {
-      return field->containing_type()->full_name() == "google.protobuf.FeatureSet";
-    });
-  }
+  FieldDescriptorSet optional_extensions;
+  CollectExtensions(*file_, options_, &extensions, &optional_extensions);
 
   // Force descriptor initialization of all dependencies.
   for (int i = 0; i < file_->dependency_count(); i++) {
@@ -484,7 +537,7 @@ void FileGenerator::GenerateDescriptorInitializationCodeForImmutable(
     }
   }
 
-  if (!extensions.empty()) {
+  if (!extensions.empty() || !optional_extensions.empty()) {
     // Must construct an ExtensionRegistry containing all existing extensions
     // and use it to parse the descriptor data again to recognize extensions.
     printer->Print(
@@ -501,6 +554,25 @@ void FileGenerator::GenerateDescriptorInitializationCodeForImmutable(
           "private static void _clinit_autosplit_dinit_$method_num$(\n"
           "    com.google.protobuf.ExtensionRegistry registry) {\n");
     }
+    for (const FieldDescriptor* field : optional_extensions) {
+      std::unique_ptr<ExtensionGenerator> generator(
+          generator_factory_->NewExtensionGenerator(field));
+      printer->Emit({{"scope", field->extension_scope() != nullptr
+                                   ? name_resolver_->GetImmutableClassName(
+                                         field->extension_scope())
+                                   : name_resolver_->GetImmutableClassName(
+                                         field->file())},
+                     {"name", UnderscoresToCamelCaseCheckReserved(field)}},
+                    R"java(
+                      addOptionalExtension(registry, "$scope$", "$name$");
+                    )java");
+      bytecode_estimate += 8;
+      MaybeRestartJavaMethod(
+          printer, &bytecode_estimate, &method_num,
+          "_clinit_autosplit_dinit_$method_num$(registry);\n",
+          "private static void _clinit_autosplit_dinit_$method_num$(\n"
+          "    com.google.protobuf.ExtensionRegistry registry) {\n");
+    }
     printer->Print(
         "com.google.protobuf.Descriptors.FileDescriptor\n"
         "    .internalUpdateFileDescriptor(descriptor, registry);\n");
@@ -564,38 +636,39 @@ void FileGenerator::GenerateSiblings(
     const std::string& package_dir, GeneratorContext* context,
     std::vector<std::string>* file_list,
     std::vector<std::string>* annotation_list) {
-  if (MultipleJavaFiles(file_, immutable_api_)) {
-    for (int i = 0; i < file_->enum_type_count(); i++) {
-      std::unique_ptr<EnumGenerator> generator(
-          generator_factory_->NewEnumGenerator(file_->enum_type(i)));
-      GenerateSibling<EnumGenerator>(
-          package_dir, java_package_, file_->enum_type(i), context, file_list,
-          options_.annotate_code, annotation_list, "", generator.get(),
-          options_.opensource_runtime, &EnumGenerator::Generate);
-    }
-    for (int i = 0; i < file_->message_type_count(); i++) {
-      if (immutable_api_) {
-        GenerateSibling<MessageGenerator>(
-            package_dir, java_package_, file_->message_type(i), context,
-            file_list, options_.annotate_code, annotation_list, "OrBuilder",
-            message_generators_[i].get(), options_.opensource_runtime,
-            &MessageGenerator::GenerateInterface);
-      }
+  for (int i = 0; i < file_->enum_type_count(); i++) {
+    if (NestedInFileClass(*file_->enum_type(i), immutable_api_)) continue;
+    std::unique_ptr<EnumGenerator> generator(
+        generator_factory_->NewEnumGenerator(file_->enum_type(i)));
+    GenerateSibling<EnumGenerator>(
+        package_dir, java_package_, file_->enum_type(i), context, file_list,
+        options_.annotate_code, annotation_list, "", generator.get(),
+        options_.opensource_runtime, &EnumGenerator::Generate);
+  }
+  for (int i = 0; i < file_->message_type_count(); i++) {
+    if (NestedInFileClass(*file_->message_type(i), immutable_api_)) continue;
+    if (immutable_api_) {
       GenerateSibling<MessageGenerator>(
           package_dir, java_package_, file_->message_type(i), context,
-          file_list, options_.annotate_code, annotation_list, "",
+          file_list, options_.annotate_code, annotation_list, "OrBuilder",
           message_generators_[i].get(), options_.opensource_runtime,
-          &MessageGenerator::Generate);
+          &MessageGenerator::GenerateInterface);
     }
-    if (HasGenericServices(file_, context_->EnforceLite())) {
-      for (int i = 0; i < file_->service_count(); i++) {
-        std::unique_ptr<ServiceGenerator> generator(
-            generator_factory_->NewServiceGenerator(file_->service(i)));
-        GenerateSibling<ServiceGenerator>(
-            package_dir, java_package_, file_->service(i), context, file_list,
-            options_.annotate_code, annotation_list, "", generator.get(),
-            options_.opensource_runtime, &ServiceGenerator::Generate);
-      }
+    GenerateSibling<MessageGenerator>(
+        package_dir, java_package_, file_->message_type(i), context, file_list,
+        options_.annotate_code, annotation_list, "",
+        message_generators_[i].get(), options_.opensource_runtime,
+        &MessageGenerator::Generate);
+  }
+  if (HasGenericServices(file_, context_->EnforceLite())) {
+    for (int i = 0; i < file_->service_count(); i++) {
+      if (NestedInFileClass(*file_->service(i), immutable_api_)) continue;
+      std::unique_ptr<ServiceGenerator> generator(
+          generator_factory_->NewServiceGenerator(file_->service(i)));
+      GenerateSibling<ServiceGenerator>(
+          package_dir, java_package_, file_->service(i), context, file_list,
+          options_.annotate_code, annotation_list, "", generator.get(),
+          options_.opensource_runtime, &ServiceGenerator::Generate);
     }
   }
 }
