@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: brianolson@google.com (Brian Olson)
 //
@@ -35,10 +12,12 @@
 
 
 #if HAVE_ZLIB
-#include <google/protobuf/io/gzip_stream.h>
+#include "google/protobuf/io/gzip_stream.h"
 
-#include <google/protobuf/stubs/common.h>
-#include <google/protobuf/stubs/logging.h>
+#include "google/protobuf/stubs/common.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
+#include "google/protobuf/port.h"
 
 namespace google {
 namespace protobuf {
@@ -54,23 +33,23 @@ GzipInputStream::GzipInputStream(ZeroCopyInputStream* sub_stream, Format format,
   zcontext_.zfree = Z_NULL;
   zcontext_.opaque = Z_NULL;
   zcontext_.total_out = 0;
-  zcontext_.next_in = NULL;
+  zcontext_.next_in = nullptr;
   zcontext_.avail_in = 0;
   zcontext_.total_in = 0;
-  zcontext_.msg = NULL;
+  zcontext_.msg = nullptr;
   if (buffer_size == -1) {
     output_buffer_length_ = kDefaultBufferSize;
   } else {
     output_buffer_length_ = buffer_size;
   }
   output_buffer_ = operator new(output_buffer_length_);
-  GOOGLE_CHECK(output_buffer_ != NULL);
+  ABSL_CHECK(output_buffer_ != nullptr);
   zcontext_.next_out = static_cast<Bytef*>(output_buffer_);
   zcontext_.avail_out = output_buffer_length_;
   output_position_ = output_buffer_;
 }
 GzipInputStream::~GzipInputStream() {
-  operator delete(output_buffer_);
+  internal::SizedDelete(output_buffer_, output_buffer_length_);
   zerror_ = inflateEnd(&zcontext_);
 }
 
@@ -97,10 +76,10 @@ int GzipInputStream::Inflate(int flush) {
   } else if (zcontext_.avail_in == 0) {
     const void* in;
     int in_size;
-    bool first = zcontext_.next_in == NULL;
+    bool first = zcontext_.next_in == nullptr;
     bool ok = sub_stream_->Next(&in, &in_size);
     if (!ok) {
-      zcontext_.next_out = NULL;
+      zcontext_.next_out = nullptr;
       zcontext_.avail_out = 0;
       return Z_STREAM_END;
     }
@@ -130,7 +109,7 @@ void GzipInputStream::DoNextOutput(const void** data, int* size) {
 bool GzipInputStream::Next(const void** data, int* size) {
   bool ok = (zerror_ == Z_OK) || (zerror_ == Z_STREAM_END) ||
             (zerror_ == Z_BUF_ERROR);
-  if ((!ok) || (zcontext_.next_out == NULL)) {
+  if ((!ok) || (zcontext_.next_out == nullptr)) {
     return false;
   }
   if (zcontext_.next_out != output_position_) {
@@ -138,7 +117,7 @@ bool GzipInputStream::Next(const void** data, int* size) {
     return true;
   }
   if (zerror_ == Z_STREAM_END) {
-    if (zcontext_.next_out != NULL) {
+    if (zcontext_.next_out != nullptr) {
       // sub_stream_ may have concatenated streams to follow
       zerror_ = inflateEnd(&zcontext_);
       byte_count_ += zcontext_.total_out;
@@ -150,13 +129,13 @@ bool GzipInputStream::Next(const void** data, int* size) {
         return false;
       }
     } else {
-      *data = NULL;
+      *data = nullptr;
       *size = 0;
       return false;
     }
   }
   zerror_ = Inflate(Z_NO_FLUSH);
-  if ((zerror_ == Z_STREAM_END) && (zcontext_.next_out == NULL)) {
+  if ((zerror_ == Z_STREAM_END) && (zcontext_.next_out == nullptr)) {
     // The underlying stream's Next returned false inside Inflate.
     return false;
   }
@@ -186,8 +165,8 @@ bool GzipInputStream::Skip(int count) {
   return ok;
 }
 int64_t GzipInputStream::ByteCount() const {
-  int64 ret = byte_count_ + zcontext_.total_out;
-  if (zcontext_.next_out != NULL && output_position_ != NULL) {
+  int64_t ret = byte_count_ + zcontext_.total_out;
+  if (zcontext_.next_out != nullptr && output_position_ != nullptr) {
     ret += reinterpret_cast<uintptr_t>(zcontext_.next_out) -
            reinterpret_cast<uintptr_t>(output_position_);
   }
@@ -214,23 +193,23 @@ GzipOutputStream::GzipOutputStream(ZeroCopyOutputStream* sub_stream,
 void GzipOutputStream::Init(ZeroCopyOutputStream* sub_stream,
                             const Options& options) {
   sub_stream_ = sub_stream;
-  sub_data_ = NULL;
+  sub_data_ = nullptr;
   sub_data_size_ = 0;
 
   input_buffer_length_ = options.buffer_size;
   input_buffer_ = operator new(input_buffer_length_);
-  GOOGLE_CHECK(input_buffer_ != NULL);
+  ABSL_CHECK(input_buffer_ != nullptr);
 
   zcontext_.zalloc = Z_NULL;
   zcontext_.zfree = Z_NULL;
   zcontext_.opaque = Z_NULL;
-  zcontext_.next_out = NULL;
+  zcontext_.next_out = nullptr;
   zcontext_.avail_out = 0;
   zcontext_.total_out = 0;
-  zcontext_.next_in = NULL;
+  zcontext_.next_in = nullptr;
   zcontext_.avail_in = 0;
   zcontext_.total_in = 0;
-  zcontext_.msg = NULL;
+  zcontext_.msg = nullptr;
   // default to GZIP format
   int windowBitsFormat = 16;
   if (options.format == ZLIB) {
@@ -244,21 +223,21 @@ void GzipOutputStream::Init(ZeroCopyOutputStream* sub_stream,
 
 GzipOutputStream::~GzipOutputStream() {
   Close();
-  operator delete(input_buffer_);
+  internal::SizedDelete(input_buffer_, input_buffer_length_);
 }
 
 // private
 int GzipOutputStream::Deflate(int flush) {
   int error = Z_OK;
   do {
-    if ((sub_data_ == NULL) || (zcontext_.avail_out == 0)) {
+    if ((sub_data_ == nullptr) || (zcontext_.avail_out == 0)) {
       bool ok = sub_stream_->Next(&sub_data_, &sub_data_size_);
       if (!ok) {
-        sub_data_ = NULL;
+        sub_data_ = nullptr;
         sub_data_size_ = 0;
         return Z_BUF_ERROR;
       }
-      GOOGLE_CHECK_GT(sub_data_size_, 0);
+      ABSL_CHECK_GT(sub_data_size_, 0);
       zcontext_.next_out = static_cast<Bytef*>(sub_data_);
       zcontext_.avail_out = sub_data_size_;
     }
@@ -268,7 +247,7 @@ int GzipOutputStream::Deflate(int flush) {
     // Notify lower layer of data.
     sub_stream_->BackUp(zcontext_.avail_out);
     // We don't own the buffer anymore.
-    sub_data_ = NULL;
+    sub_data_ = nullptr;
     sub_data_size_ = 0;
   }
   return error;
@@ -293,12 +272,12 @@ bool GzipOutputStream::Next(void** data, int* size) {
     *size = input_buffer_length_;
   } else {
     // The loop in Deflate should consume all avail_in
-    GOOGLE_LOG(DFATAL) << "Deflate left bytes unconsumed";
+    ABSL_DLOG(FATAL) << "Deflate left bytes unconsumed";
   }
   return true;
 }
 void GzipOutputStream::BackUp(int count) {
-  GOOGLE_CHECK_GE(zcontext_.avail_in, count);
+  ABSL_CHECK_GE(zcontext_.avail_in, static_cast<uInt>(count));
   zcontext_.avail_in -= count;
 }
 int64_t GzipOutputStream::ByteCount() const {
