@@ -84,6 +84,13 @@ absl::Status JsonToBinaryString(google::protobuf::util::TypeResolver* resolver,
 
 absl::Status MessageToJsonString(const Message& message, std::string* output,
                                  const PrintOptions& options) {
+  io::StringOutputStream out(output);
+  return MessageToJsonStream(message, &out, options);
+}
+
+absl::Status MessageToJsonStream(const Message& message,
+                                 io::ZeroCopyOutputStream* json_output,
+                                 const PrintOptions& options) {
   google::protobuf::json_internal::WriterOptions opts;
   opts.add_whitespace = options.add_whitespace;
   opts.preserve_proto_field_names = options.preserve_proto_field_names;
@@ -95,11 +102,18 @@ absl::Status MessageToJsonString(const Message& message, std::string* output,
   // TODO: Drop this setting.
   opts.allow_legacy_syntax = true;
 
-  return google::protobuf::json_internal::MessageToJsonString(message, output, opts);
+  return google::protobuf::json_internal::MessageToJsonStream(message, json_output, opts);
 }
 
 absl::Status JsonStringToMessage(absl::string_view input, Message* message,
                                  const ParseOptions& options) {
+  io::ArrayInputStream input_stream(input.data(), input.size());
+  return JsonStreamToMessage(&input_stream, message, options);
+}
+
+absl::Status JsonStreamToMessage(io::ZeroCopyInputStream* input,
+                                 Message* message,
+                                 const ParseOptions& options) {
   google::protobuf::json_internal::ParseOptions opts;
   opts.ignore_unknown_fields = options.ignore_unknown_fields;
   opts.case_insensitive_enum_parsing = options.case_insensitive_enum_parsing;
@@ -107,8 +121,10 @@ absl::Status JsonStringToMessage(absl::string_view input, Message* message,
   // TODO: Drop this setting.
   opts.allow_legacy_syntax = true;
 
-  return google::protobuf::json_internal::JsonStringToMessage(input, message, opts);
+  return google::protobuf::json_internal::JsonStreamToMessage(input, message, opts);
 }
 }  // namespace json
 }  // namespace protobuf
 }  // namespace google
+
+#include "google/protobuf/port_undef.inc"
