@@ -369,7 +369,7 @@ void MessageDifferencer::CheckRepeatedFieldComparisons(
   ABSL_CHECK(field->is_repeated())
       << "Field must be repeated: " << field->full_name();
   const MapKeyComparator* key_comparator = GetMapKeyComparator(field);
-  ABSL_CHECK(key_comparator == NULL)
+  ABSL_CHECK(key_comparator == nullptr)
       << "Cannot treat this repeated field as both MAP and " << new_comparison
       << " for comparison.  Field name is: " << field->full_name();
 }
@@ -407,8 +407,7 @@ void MessageDifferencer::TreatAsMap(const FieldDescriptor* field,
       << key->full_name()
       << " must be a direct subfield within the repeated field "
       << field->full_name() << ", not " << key->containing_type()->full_name();
-  ABSL_CHECK(repeated_field_comparisons_.find(field) ==
-             repeated_field_comparisons_.end())
+  ABSL_CHECK(!repeated_field_comparisons_.contains(field))
       << "Cannot treat the same field as both "
       << repeated_field_comparisons_[field]
       << " and MAP. Field name is: " << field->full_name();
@@ -455,8 +454,7 @@ void MessageDifferencer::TreatAsMapWithMultipleFieldPathsAsKey(
       }
     }
   }
-  ABSL_CHECK(repeated_field_comparisons_.find(field) ==
-             repeated_field_comparisons_.end())
+  ABSL_CHECK(!repeated_field_comparisons_.contains(field))
       << "Cannot treat the same field as both "
       << repeated_field_comparisons_[field]
       << " and MAP. Field name is: " << field->full_name();
@@ -470,8 +468,7 @@ void MessageDifferencer::TreatAsMapUsingKeyComparator(
     const FieldDescriptor* field, const MapKeyComparator* key_comparator) {
   ABSL_CHECK(field->is_repeated())
       << "Field must be repeated: " << field->full_name();
-  ABSL_CHECK(repeated_field_comparisons_.find(field) ==
-             repeated_field_comparisons_.end())
+  ABSL_CHECK(!repeated_field_comparisons_.contains(field))
       << "Cannot treat the same field as both "
       << repeated_field_comparisons_[field]
       << " and MAP. Field name is: " << field->full_name();
@@ -503,7 +500,7 @@ void MessageDifferencer::ReportDifferencesTo(Reporter* reporter) {
   // If an output string is set, clear it to prevent
   // it superseding the specified reporter.
   if (output_string_) {
-    output_string_ = NULL;
+    output_string_ = nullptr;
   }
 
   reporter_ = reporter;
@@ -513,11 +510,11 @@ bool MessageDifferencer::FieldBefore(const FieldDescriptor* field1,
                                      const FieldDescriptor* field2) {
   // Handle sentinel values (i.e. make sure NULLs are always ordered
   // at the end of the list).
-  if (field1 == NULL) {
+  if (field1 == nullptr) {
     return false;
   }
 
-  if (field2 == NULL) {
+  if (field2 == nullptr) {
     return true;
   }
 
@@ -548,7 +545,7 @@ bool MessageDifferencer::Compare(const Message& message1,
     reporter.SetMessages(message1, message2);
     reporter_ = &reporter;
     result = Compare(message1, message2, false, &parent_fields);
-    reporter_ = NULL;
+    reporter_ = nullptr;
   } else {
     result = Compare(message1, message2, false, &parent_fields);
   }
@@ -596,7 +593,7 @@ bool MessageDifferencer::CompareWithFields(
     result = CompareRequestedFieldsUsingSettings(
         message1, message2, false, message1_fields, message2_fields,
         &parent_fields);
-    reporter_ = NULL;
+    reporter_ = nullptr;
   } else {
     result = CompareRequestedFieldsUsingSettings(
         message1, message2, false, message1_fields, message2_fields,
@@ -634,7 +631,7 @@ bool MessageDifferencer::Compare(const Message& message1,
         reflection2->GetUnknownFields(message2);
     if (!CompareUnknownFields(message1, message2, unknown_field_set1,
                               unknown_field_set2, parent_fields)) {
-      if (reporter_ == NULL) {
+      if (reporter_ == nullptr) {
         return false;
       }
       unknown_compare_result = false;
@@ -844,7 +841,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
     const FieldDescriptor* field2 = message2_fields[field_index2];
 
     // Once we have reached sentinel values, we are done the comparison.
-    if (field1 == NULL && field2 == NULL) {
+    if (field1 == nullptr && field2 == nullptr) {
       break;
     }
 
@@ -854,7 +851,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
       if (IsIgnored(message1, message2, field1, *parent_fields)) {
         // We are ignoring field1. Report the ignore and move on to
         // the next field in message1_fields.
-        if (reporter_ != NULL) {
+        if (reporter_ != nullptr) {
           SpecificField& specific_field = PushSpecificField(parent_fields);
           specific_field.message1 = &message1;
           specific_field.message2 = &message2;
@@ -869,8 +866,8 @@ bool MessageDifferencer::CompareWithFieldsInternal(
         continue;
       }
 
-      if (reporter_ != NULL) {
-        assert(field1 != NULL);
+      if (reporter_ != nullptr) {
+        assert(field1 != nullptr);
         int count = field1->is_repeated()
                         ? reflection1->FieldSize(message1, field1)
                         : 1;
@@ -909,7 +906,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
       if (ignore_field) {
         // We are ignoring field2. Report the ignore and move on to
         // the next field in message2_fields.
-        if (reporter_ != NULL) {
+        if (reporter_ != nullptr) {
           SpecificField& specific_field = PushSpecificField(parent_fields);
           specific_field.message1 = &message1;
           specific_field.message2 = &message2;
@@ -924,7 +921,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
         continue;
       }
 
-      if (reporter_ != NULL) {
+      if (reporter_ != nullptr) {
         int count = field2->is_repeated()
                         ? reflection2->FieldSize(message2, field2)
                         : 1;
@@ -964,7 +961,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
     // field, so we can now compare the values.
     if (IsIgnored(message1, message2, field1, *parent_fields)) {
       // Ignore this field. Report and move on.
-      if (reporter_ != NULL) {
+      if (reporter_ != nullptr) {
         SpecificField& specific_field = PushSpecificField(parent_fields);
         specific_field.message1 = &message1;
         specific_field.message2 = &message2;
@@ -982,7 +979,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
     }
 
     bool fieldDifferent = false;
-    assert(field1 != NULL);
+    assert(field1 != nullptr);
     if (field1->is_map()) {
       fieldDifferent = !CompareMapField(message1, message2, unpacked_any,
                                         field1, parent_fields);
@@ -1043,10 +1040,10 @@ bool MessageDifferencer::IsMatch(
   Reporter* backup_reporter = reporter_;
   std::string* output_string = output_string_;
   reporter_ = reporter;
-  output_string_ = NULL;
+  output_string_ = nullptr;
   bool match;
 
-  if (key_comparator == NULL) {
+  if (key_comparator == nullptr) {
     match = CompareFieldValueUsingParentFields(
         *message1, *message2, unpacked_any, repeated_field, index1, index2,
         &current_parent_fields);
@@ -1083,8 +1080,7 @@ bool MessageDifferencer::CompareMapFieldByMapReflection(
     DefaultFieldComparator* comparator) {
   ABSL_DCHECK_EQ(nullptr, reporter_);
   ABSL_DCHECK(map_field->is_map());
-  ABSL_DCHECK(map_field_key_comparator_.find(map_field) ==
-              map_field_key_comparator_.end());
+  ABSL_DCHECK(!map_field_key_comparator_.contains(map_field));
   ABSL_DCHECK_EQ(repeated_field_comparison_, AS_LIST);
   const Reflection* reflection1 = message1.GetReflection();
   const Reflection* reflection2 = message2.GetReflection();
@@ -1099,10 +1095,8 @@ bool MessageDifferencer::CompareMapFieldByMapReflection(
   }
 
   // First pass: check whether the same keys are present.
-  for (MapIterator it = reflection1->MapBegin(const_cast<Message*>(&message1),
-                                              map_field),
-                   it_end = reflection1->MapEnd(const_cast<Message*>(&message1),
-                                                map_field);
+  for (ConstMapIterator it = reflection1->ConstMapBegin(&message1, map_field),
+                        it_end = reflection1->ConstMapEnd(&message1, map_field);
        it != it_end; ++it) {
     if (!reflection2->ContainsMapKey(message2, map_field, it.GetKey())) {
       return false;
@@ -1114,10 +1108,9 @@ bool MessageDifferencer::CompareMapFieldByMapReflection(
   switch (val_des->cpp_type()) {
 #define HANDLE_TYPE(CPPTYPE, METHOD, COMPAREMETHOD)                           \
   case FieldDescriptor::CPPTYPE_##CPPTYPE: {                                  \
-    for (MapIterator it = reflection1->MapBegin(                              \
-                         const_cast<Message*>(&message1), map_field),         \
-                     it_end = reflection1->MapEnd(                            \
-                         const_cast<Message*>(&message1), map_field);         \
+    for (ConstMapIterator                                                     \
+             it = reflection1->ConstMapBegin(&message1, map_field),           \
+             it_end = reflection1->ConstMapEnd(&message1, map_field);         \
          it != it_end; ++it) {                                                \
       MapValueConstRef value2;                                                \
       reflection2->LookupMapValue(message2, map_field, it.GetKey(), &value2); \
@@ -1140,11 +1133,9 @@ bool MessageDifferencer::CompareMapFieldByMapReflection(
     HANDLE_TYPE(ENUM, EnumValue, Int32);
 #undef HANDLE_TYPE
     case FieldDescriptor::CPPTYPE_MESSAGE: {
-      for (MapIterator it = reflection1->MapBegin(
-               const_cast<Message*>(&message1), map_field);
-           it !=
-           reflection1->MapEnd(const_cast<Message*>(&message1), map_field);
-           ++it) {
+      for (ConstMapIterator it =
+               reflection1->ConstMapBegin(&message1, map_field);
+           it != reflection1->ConstMapEnd(&message1, map_field); ++it) {
         if (!reflection2->ContainsMapKey(message2, map_field, it.GetKey())) {
           return false;
         }
@@ -1188,8 +1179,7 @@ bool MessageDifferencer::CompareMapField(
       // TODO: Add support for reporter
       reporter_ == nullptr &&
       // Users didn't set custom map field key comparator
-      map_field_key_comparator_.find(repeated_field) ==
-          map_field_key_comparator_.end() &&
+      !map_field_key_comparator_.contains(repeated_field) &&
       // Users didn't set repeated field comparison
       repeated_field_comparison_ == AS_LIST &&
       // Users didn't set their own FieldComparator implementation
@@ -1241,11 +1231,11 @@ bool MessageDifferencer::CompareRepeatedRep(
   // If the field is not treated as subset and no detailed reports is needed,
   // we do a quick check on the number of the elements to avoid unnecessary
   // comparison.
-  if (count1 != count2 && reporter_ == NULL && !treated_as_subset) {
+  if (count1 != count2 && reporter_ == nullptr && !treated_as_subset) {
     return false;
   }
   // A match can never be found if message1 has more items than message2.
-  if (count1 > count2 && reporter_ == NULL) {
+  if (count1 > count2 && reporter_ == nullptr) {
     return false;
   }
 
@@ -1331,18 +1321,18 @@ bool MessageDifferencer::CompareRepeatedRep(
     // no reporter is present. Note that ReportModified, ReportMoved, and
     // ReportMatched are all mutually exclusive.
     if (!result) {
-      if (reporter_ == NULL) return false;
+      if (reporter_ == nullptr) return false;
       parent_fields->push_back(specific_field);
       reporter_->ReportModified(message1, message2, *parent_fields);
       parent_fields->pop_back();
       fieldDifferent = true;
-    } else if (reporter_ != NULL &&
+    } else if (reporter_ != nullptr &&
                specific_field.index != specific_field.new_index &&
                !specific_field.field->is_map() && report_moves_) {
       parent_fields->push_back(specific_field);
       reporter_->ReportMoved(message1, message2, *parent_fields);
       parent_fields->pop_back();
-    } else if (report_matches_ && reporter_ != NULL) {
+    } else if (report_matches_ && reporter_ != nullptr) {
       parent_fields->push_back(specific_field);
       reporter_->ReportMatched(message1, message2, *parent_fields);
       parent_fields->pop_back();
@@ -1357,7 +1347,7 @@ bool MessageDifferencer::CompareRepeatedRep(
       fieldDifferent = true;
     }
 
-    if (reporter_ == NULL) continue;
+    if (reporter_ == nullptr) continue;
     specific_field.index = i;
     AddSpecificNewIndex(&specific_field, message2, repeated_field, i);
     parent_fields->push_back(specific_field);
@@ -1368,7 +1358,7 @@ bool MessageDifferencer::CompareRepeatedRep(
   for (int i = 0; i < count1; ++i) {
     if (!simple_list && match_list1[i] != -1) continue;
     if (simple_list && i < count2) continue;
-    assert(reporter_ != NULL);
+    assert(reporter_ != nullptr);
     AddSpecificIndex(&specific_field, message1, repeated_field, i);
     parent_fields->push_back(specific_field);
     reporter_->ReportDeleted(message1, message2, *parent_fields);
@@ -1411,7 +1401,7 @@ bool MessageDifferencer::CompareFieldValueUsingParentFields(
             : reflection2->GetMessage(message2, field);
 
     // parent_fields is used in calls to Reporter methods.
-    if (parent_fields != NULL) {
+    if (parent_fields != nullptr) {
       // Append currently compared field to the end of parent_fields.
       SpecificField& specific_field = PushSpecificField(parent_fields);
       specific_field.message1 = &message1;
@@ -1445,8 +1435,7 @@ bool MessageDifferencer::CheckPathChanged(
 
 bool MessageDifferencer::IsTreatedAsSet(const FieldDescriptor* field) {
   if (!field->is_repeated()) return false;
-  if (repeated_field_comparisons_.find(field) !=
-      repeated_field_comparisons_.end()) {
+  if (repeated_field_comparisons_.contains(field)) {
     return repeated_field_comparisons_[field] == AS_SET;
   }
   return GetMapKeyComparator(field) == nullptr &&
@@ -1455,8 +1444,7 @@ bool MessageDifferencer::IsTreatedAsSet(const FieldDescriptor* field) {
 
 bool MessageDifferencer::IsTreatedAsSmartSet(const FieldDescriptor* field) {
   if (!field->is_repeated()) return false;
-  if (repeated_field_comparisons_.find(field) !=
-      repeated_field_comparisons_.end()) {
+  if (repeated_field_comparisons_.contains(field)) {
     return repeated_field_comparisons_[field] == AS_SMART_SET;
   }
   return GetMapKeyComparator(field) == nullptr &&
@@ -1465,8 +1453,7 @@ bool MessageDifferencer::IsTreatedAsSmartSet(const FieldDescriptor* field) {
 
 bool MessageDifferencer::IsTreatedAsSmartList(const FieldDescriptor* field) {
   if (!field->is_repeated()) return false;
-  if (repeated_field_comparisons_.find(field) !=
-      repeated_field_comparisons_.end()) {
+  if (repeated_field_comparisons_.contains(field)) {
     return repeated_field_comparisons_[field] == AS_SMART_LIST;
   }
   return GetMapKeyComparator(field) == nullptr &&
@@ -1475,14 +1462,14 @@ bool MessageDifferencer::IsTreatedAsSmartList(const FieldDescriptor* field) {
 
 bool MessageDifferencer::IsTreatedAsSubset(const FieldDescriptor* field) {
   return scope_ == PARTIAL &&
-         (IsTreatedAsSet(field) || GetMapKeyComparator(field) != NULL);
+         (IsTreatedAsSet(field) || GetMapKeyComparator(field) != nullptr);
 }
 
 bool MessageDifferencer::IsIgnored(
     const Message& message1, const Message& message2,
     const FieldDescriptor* field,
     const std::vector<SpecificField>& parent_fields) {
-  if (ignored_fields_.find(field) != ignored_fields_.end()) {
+  if (ignored_fields_.contains(field)) {
     return true;
   }
   for (const auto& criteria : ignore_criteria_) {
@@ -1553,12 +1540,12 @@ bool MessageDifferencer::UnpackAnyField::UnpackAny(
   const Descriptor* desc =
       any.GetDescriptor()->file()->pool()->FindMessageTypeByName(
           full_type_name);
-  if (desc == NULL) {
+  if (desc == nullptr) {
     return false;
   }
 
-  if (dynamic_message_factory_ == NULL) {
-    dynamic_message_factory_.reset(new DynamicMessageFactory());
+  if (dynamic_message_factory_ == nullptr) {
+    dynamic_message_factory_ = std::make_unique<DynamicMessageFactory>();
   }
   data->reset(dynamic_message_factory_->GetPrototype(desc)->New());
   std::string serialized_value = reflection->GetString(any, value_field);
@@ -1610,7 +1597,7 @@ bool MessageDifferencer::CompareUnknownFields(
   // current_repeated points at the first field in this range, and
   // current_repeated_start{1,2} are the indexes of the first field in the
   // range within fields1 and fields2.
-  const UnknownField* current_repeated = NULL;
+  const UnknownField* current_repeated = nullptr;
   int current_repeated_start1 = 0;
   int current_repeated_start2 = 0;
 
@@ -1680,7 +1667,7 @@ bool MessageDifferencer::CompareUnknownFields(
       }
     }
 
-    if (current_repeated == NULL ||
+    if (current_repeated == nullptr ||
         focus_field->number() != current_repeated->number() ||
         focus_field->type() != current_repeated->type()) {
       // We've started a new repeated field.
@@ -1689,7 +1676,7 @@ bool MessageDifferencer::CompareUnknownFields(
       current_repeated_start2 = index2;
     }
 
-    if (change_type == NO_CHANGE && reporter_ == NULL) {
+    if (change_type == NO_CHANGE && reporter_ == nullptr) {
       // Fields were already compared and matched and we have no reporter.
       ++index1;
       ++index2;
@@ -1724,7 +1711,7 @@ bool MessageDifferencer::CompareUnknownFields(
 
     if (IsUnknownFieldIgnored(message1, message2, specific_field,
                               *parent_field)) {
-      if (report_ignores_ && reporter_ != NULL) {
+      if (report_ignores_ && reporter_ != nullptr) {
         parent_field->push_back(specific_field);
         reporter_->ReportUnknownFieldIgnored(message1, message2, *parent_field);
         parent_field->pop_back();
@@ -1736,7 +1723,7 @@ bool MessageDifferencer::CompareUnknownFields(
 
     if (change_type == ADDITION || change_type == DELETION ||
         change_type == MODIFICATION) {
-      if (reporter_ == NULL) {
+      if (reporter_ == nullptr) {
         // We found a difference and we have no reporter.
         return false;
       }
@@ -1763,7 +1750,7 @@ bool MessageDifferencer::CompareUnknownFields(
         if (!CompareUnknownFields(
                 message1, message2, fields1[index1].second->group(),
                 fields2[index2].second->group(), parent_field)) {
-          if (reporter_ == NULL) return false;
+          if (reporter_ == nullptr) return false;
           is_different = true;
           reporter_->ReportModified(message1, message2, *parent_field);
         }
@@ -1943,7 +1930,7 @@ bool MessageDifferencer::MatchRepeatedFieldIndices(
   // perform further comparisons, turn off reporting here and re-enable it
   // before returning.
   Reporter* reporter = reporter_;
-  reporter_ = NULL;
+  reporter_ = nullptr;
   NumDiffsReporter num_diffs_reporter;
   std::vector<int32_t> num_diffs_list1;
   if (is_treated_as_smart_set) {
@@ -2126,7 +2113,7 @@ void MessageDifferencer::StreamReporter::PrintPath(
     if (i > 0) {
       printer_->Print(".");
     }
-    if (specific_field.field != NULL) {
+    if (specific_field.field != nullptr) {
       if (specific_field.field->is_extension()) {
         printer_->Print("($name$)", "name", specific_field.field->full_name());
       } else {
@@ -2158,7 +2145,7 @@ void MessageDifferencer::StreamReporter::PrintValue(
     bool left_side) {
   const SpecificField& specific_field = field_path.back();
   const FieldDescriptor* field = specific_field.field;
-  if (field != NULL) {
+  if (field != nullptr) {
     std::string output;
     int index = left_side ? specific_field.index : specific_field.new_index;
     if (field->cpp_type() == FieldDescriptor::CPPTYPE_MESSAGE) {
@@ -2207,7 +2194,7 @@ void MessageDifferencer::StreamReporter::PrintValue(
 
 void MessageDifferencer::StreamReporter::PrintUnknownFieldValue(
     const UnknownField* unknown_field) {
-  ABSL_CHECK(unknown_field != NULL) << " Cannot print NULL unknown_field.";
+  ABSL_CHECK(unknown_field != nullptr) << " Cannot print NULL unknown_field.";
 
   std::string output;
   switch (unknown_field->type()) {
@@ -2237,7 +2224,7 @@ void MessageDifferencer::StreamReporter::PrintUnknownFieldValue(
 }
 
 void MessageDifferencer::StreamReporter::Print(const std::string& str) {
-  printer_->Print(str.c_str());
+  printer_->Print(str);
 }
 
 void MessageDifferencer::StreamReporter::PrintMapKey(
@@ -2293,7 +2280,7 @@ void MessageDifferencer::StreamReporter::ReportDeleted(
 void MessageDifferencer::StreamReporter::ReportModified(
     const Message& message1, const Message& message2,
     const std::vector<SpecificField>& field_path) {
-  if (!report_modified_aggregates_ && field_path.back().field == NULL) {
+  if (!report_modified_aggregates_ && field_path.back().field == nullptr) {
     if (field_path.back().unknown_field_type == UnknownField::TYPE_GROUP) {
       // Any changes to the subfields have already been printed.
       return;
